const { app, BrowserWindow, ipcMain, dialog } = require('electron');
const path = require('path');
const fs = require('fs').promises;
const isDev = process.env.NODE_ENV !== 'production';

process.on('uncaughtException', (error) => {
  console.error('Uncaught Exception:', error);
});

process.on('unhandledRejection', (error) => {
  console.error('Unhandled Rejection:', error);
});

console.log('Electron process starting...');
console.log('Current working directory:', process.cwd());
console.log('isDev:', isDev);
console.log('electron version:', process.versions.electron);
console.log('chrome version:', process.versions.chrome);
console.log('node version:', process.versions.node);

let mainWindow = null;

function createWindow() {
  console.log('Creating window...');
  
  mainWindow = new BrowserWindow({
    width: 1200,
    height: 800,
    show: false,
    webPreferences: {
      nodeIntegration: false,
      contextIsolation: true,
      preload: path.join(__dirname, 'preload.js'),
      enableRemoteModule: false,
      sandbox: true
    }
  });

  const devServerUrl = 'http://localhost:5173';
  
  if (isDev) {
    console.log('Loading URL in dev mode:', devServerUrl);
    mainWindow.loadURL(devServerUrl).catch(err => {
      console.error('Failed to load dev server:', err);
      app.quit();
    });
    mainWindow.webContents.openDevTools();
  } else {
    const indexPath = path.join(__dirname, '../dist/index.html');
    console.log('Loading file in prod mode:', indexPath);
    mainWindow.loadFile(indexPath).catch(err => {
      console.error('Failed to load index file:', err);
      app.quit();
    });
  }

  mainWindow.webContents.on('did-fail-load', (event, errorCode, errorDescription) => {
    console.error('Failed to load:', errorCode, errorDescription);
  });

  mainWindow.once('ready-to-show', () => {
    console.log('Window ready to show');
    mainWindow.show();
  });

  mainWindow.on('closed', () => {
    console.log('Window closed');
    mainWindow = null;
  });
}

// 处理保存文件
ipcMain.handle('save-file', async (event, content) => {
  try {
    const { filePath } = await dialog.showSaveDialog({
      title: 'Save Markdown File',
      defaultPath: path.join(app.getPath('documents'), 'untitled.md'),
      filters: [
        { name: 'Markdown Files', extensions: ['md'] },
        { name: 'All Files', extensions: ['*'] }
      ]
    });

    if (filePath) {
      await fs.writeFile(filePath, content, 'utf8');
      return { success: true, filePath };
    }
    return { success: false, error: 'No file selected' };
  } catch (error) {
    return { success: false, error: error.message };
  }
});

// 处理导出 PDF
ipcMain.handle('export-pdf', async (event) => {
  try {
    const win = BrowserWindow.fromWebContents(event.sender);
    
    const { filePath } = await dialog.showSaveDialog(win, {
      title: 'Export PDF',
      defaultPath: path.join(app.getPath('documents'), 'untitled.pdf'),
      filters: [
        { name: 'PDF Files', extensions: ['pdf'] }
      ]
    });

    if (!filePath) {
      return { success: false, error: 'No file selected' };
    }

    // 获取预览内容的 HTML
    const previewHTML = await event.sender.executeJavaScript(`
      document.querySelector('.preview-for-pdf').innerHTML
    `);

    // 创建一个临时窗口来渲染 PDF
    const tempWin = new BrowserWindow({
      width: 800,
      height: 600,
      show: false
    });

    // 加载一个包含预览内容的临时 HTML
    await tempWin.loadURL(`data:text/html;charset=utf-8,
      <!DOCTYPE html>
      <html>
        <head>
          <style>
            body {
              font-family: -apple-system, BlinkMacSystemFont, 'Segoe UI', Roboto, 'Helvetica Neue', Arial, sans-serif;
              line-height: 1.6;
              color: #333;
              padding: 20px;
            }
            pre {
              background-color: #f8f8f8;
              padding: 1rem;
              border-radius: 4px;
              overflow-x: auto;
            }
            code {
              background-color: #f8f8f8;
              padding: 0.2em 0.4em;
              border-radius: 3px;
              font-family: 'SFMono-Regular', Consolas, 'Liberation Mono', Menlo, Courier, monospace;
            }
            blockquote {
              margin: 1rem 0;
              padding-left: 1rem;
              border-left: 4px solid #e0e0e0;
              color: #666;
            }
            table {
              border-collapse: collapse;
              width: 100%;
              margin: 1rem 0;
            }
            th, td {
              border: 1px solid #e0e0e0;
              padding: 8px;
              text-align: left;
            }
            th {
              background-color: #f8f8f8;
            }
          </style>
        </head>
        <body>
          ${previewHTML}
        </body>
      </html>
    `);

    // 导出为 PDF
    const pdfData = await tempWin.webContents.printToPDF({
      printBackground: true,
      margin: {
        top: 36,
        bottom: 36,
        left: 36,
        right: 36
      }
    });

    // 保存 PDF 文件
    await fs.writeFile(filePath, pdfData);

    // 关闭临时窗口
    tempWin.close();

    return { success: true, filePath };
  } catch (error) {
    console.error('Error exporting PDF:', error);
    return { success: false, error: error.message };
  }
});

app.whenReady().then(() => {
  console.log('App ready, creating window...');
  createWindow();
}).catch(err => {
  console.error('Failed to initialize app:', err);
});

app.on('window-all-closed', () => {
  console.log('All windows closed');
  if (process.platform !== 'darwin') {
    app.quit();
  }
});

app.on('activate', () => {
  console.log('App activated');
  if (mainWindow === null) {
    createWindow();
  }
});
