import React, { useState, useRef, useCallback } from 'react';
import Split from 'react-split';
import Editor from '@monaco-editor/react';
import { marked } from 'marked';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import {
  faBold,
  faItalic,
  faStrikethrough,
  faCode,
  faFileCode,
  faLink,
  faImage,
  faListUl,
  faListOl,
  faQuoteRight,
  faTable,
  faMinus,
  faColumns,
  faEdit,
  faEye,
  faSave,
  faFilePdf
} from '@fortawesome/free-solid-svg-icons';

declare global {
  interface Window {
    electron: {
      ipcRenderer: {
        invoke(channel: string, ...args: any[]): Promise<any>;
      };
    };
  }
}

type ViewMode = 'split' | 'edit' | 'preview';

const App: React.FC = () => {
  const editorRef = useRef<any>(null);
  const [markdown, setMarkdown] = useState<string>('');
  const [viewMode, setViewMode] = useState<ViewMode>('split');

  const insertAtCursor = (before: string, after: string = '') => {
    const editor = editorRef.current;
    if (!editor) return;

    const selection = editor.getSelection();
    const selectedText = editor.getModel().getValueInRange(selection);
    const newText = selectedText ? before + selectedText + after : before + after;

    editor.executeEdits('', [{
      range: selection,
      text: newText,
      forceMoveMarkers: true
    }]);

    // 设置新的光标位置
    if (!selectedText) {
      const newPosition = editor.getPosition();
      newPosition.column -= after.length;
      editor.setPosition(newPosition);
    }

    editor.focus();
  };

  const insertAtLineStart = (prefix: string) => {
    const editor = editorRef.current;
    if (!editor) return;

    const selection = editor.getSelection();
    const startLine = selection.startLineNumber;
    const endLine = selection.endLineNumber;

    for (let i = startLine; i <= endLine; i++) {
      const lineContent = editor.getModel().getLineContent(i);
      editor.executeEdits('', [{
        range: {
          startLineNumber: i,
          startColumn: 1,
          endLineNumber: i,
          endColumn: 1
        },
        text: prefix,
        forceMoveMarkers: true
      }]);
    }

    editor.focus();
  };

  const handleSave = async () => {
    try {
      const result = await window.electron.ipcRenderer.invoke('save-file', markdown);
      if (result.success) {
        console.log('File saved successfully:', result.filePath);
      } else {
        console.error('Failed to save file:', result.error);
      }
    } catch (error) {
      console.error('Error saving file:', error);
    }
  };

  const handleExportPDF = async () => {
    try {
      const result = await window.electron.ipcRenderer.invoke('export-pdf');
      if (result.success) {
        console.log('PDF exported successfully:', result.filePath);
      } else {
        console.error('Failed to export PDF:', result.error);
      }
    } catch (error) {
      console.error('Error exporting PDF:', error);
    }
  };

  const handleToolbarAction = useCallback((action: string) => {
    switch (action) {
      case 'bold':
        insertAtCursor('**', '**');
        break;
      case 'italic':
        insertAtCursor('*', '*');
        break;
      case 'strikethrough':
        insertAtCursor('~~', '~~');
        break;
      case 'code':
        insertAtCursor('`', '`');
        break;
      case 'codeblock':
        insertAtCursor('```\n', '\n```');
        break;
      case 'link':
        insertAtCursor('[', '](url)');
        break;
      case 'image':
        insertAtCursor('![alt text](', ')');
        break;
      case 'bulletList':
        insertAtLineStart('- ');
        break;
      case 'numberedList':
        insertAtLineStart('1. ');
        break;
      case 'quote':
        insertAtLineStart('> ');
        break;
      case 'h1':
        insertAtLineStart('# ');
        break;
      case 'h2':
        insertAtLineStart('## ');
        break;
      case 'h3':
        insertAtLineStart('### ');
        break;
      case 'table':
        insertAtCursor('| Header | Header |\n| --- | --- |\n| Cell | Cell |');
        break;
      case 'hr':
        insertAtCursor('---\n');
        break;
      case 'save':
        handleSave();
        break;
      case 'export-pdf':
        handleExportPDF();
        break;
    }
  }, [markdown]);

  const handleEditorDidMount = (editor: any) => {
    editorRef.current = editor;
  };

  const handleEditorChange = (value: string | undefined) => {
    if (value !== undefined) {
      setMarkdown(value);
    }
  };

  return (
    <div className="app">
      <div className="toolbar">
        <div className="toolbar-group">
          <div className="toolbar-section">
            <button onClick={() => handleToolbarAction('bold')} title="Bold">
              <FontAwesomeIcon icon={faBold} />
            </button>
            <button onClick={() => handleToolbarAction('italic')} title="Italic">
              <FontAwesomeIcon icon={faItalic} />
            </button>
            <button onClick={() => handleToolbarAction('strikethrough')} title="Strikethrough">
              <FontAwesomeIcon icon={faStrikethrough} />
            </button>
          </div>

          <div className="toolbar-divider" />

          <div className="toolbar-section">
            <button onClick={() => handleToolbarAction('code')} title="Inline Code">
              <FontAwesomeIcon icon={faCode} />
            </button>
            <button onClick={() => handleToolbarAction('codeblock')} title="Code Block">
              <FontAwesomeIcon icon={faFileCode} />
            </button>
          </div>

          <div className="toolbar-divider" />

          <div className="toolbar-section">
            <button onClick={() => handleToolbarAction('link')} title="Link">
              <FontAwesomeIcon icon={faLink} />
            </button>
            <button onClick={() => handleToolbarAction('image')} title="Image">
              <FontAwesomeIcon icon={faImage} />
            </button>
          </div>

          <div className="toolbar-divider" />

          <div className="toolbar-section">
            <button onClick={() => handleToolbarAction('bulletList')} title="Bullet List">
              <FontAwesomeIcon icon={faListUl} />
            </button>
            <button onClick={() => handleToolbarAction('numberedList')} title="Numbered List">
              <FontAwesomeIcon icon={faListOl} />
            </button>
            <button onClick={() => handleToolbarAction('quote')} title="Quote">
              <FontAwesomeIcon icon={faQuoteRight} />
            </button>
          </div>

          <div className="toolbar-divider" />

          <div className="toolbar-section">
            <button onClick={() => handleToolbarAction('h1')} title="Heading 1">H1</button>
            <button onClick={() => handleToolbarAction('h2')} title="Heading 2">H2</button>
            <button onClick={() => handleToolbarAction('h3')} title="Heading 3">H3</button>
          </div>

          <div className="toolbar-divider" />

          <div className="toolbar-section">
            <button onClick={() => handleToolbarAction('table')} title="Table">
              <FontAwesomeIcon icon={faTable} />
            </button>
            <button onClick={() => handleToolbarAction('hr')} title="Horizontal Rule">
              <FontAwesomeIcon icon={faMinus} />
            </button>
          </div>
        </div>

        <div className="toolbar-group toolbar-group-right">
          <button
            onClick={() => setViewMode('split')}
            className={viewMode === 'split' ? 'active' : ''}
            title="Split View"
          >
            <FontAwesomeIcon icon={faColumns} />
          </button>
          <button
            onClick={() => setViewMode('edit')}
            className={viewMode === 'edit' ? 'active' : ''}
            title="Editor Only"
          >
            <FontAwesomeIcon icon={faEdit} />
          </button>
          <button
            onClick={() => setViewMode('preview')}
            className={viewMode === 'preview' ? 'active' : ''}
            title="Preview Only"
          >
            <FontAwesomeIcon icon={faEye} />
          </button>
          <div className="toolbar-divider" />
          <button onClick={() => handleToolbarAction('save')} title="Save">
            <FontAwesomeIcon icon={faSave} />
          </button>
          <button onClick={() => handleToolbarAction('export-pdf')} title="Export PDF">
            <FontAwesomeIcon icon={faFilePdf} />
          </button>
        </div>
      </div>

      <div className="content">
        {viewMode === 'split' ? (
          <Split className="split" sizes={[50, 50]}>
            <div className="editor-container">
              <Editor
                height="100%"
                defaultLanguage="markdown"
                value={markdown}
                onChange={handleEditorChange}
                onMount={handleEditorDidMount}
                theme="vs-light"
                options={{
                  minimap: { enabled: false },
                  fontSize: 14,
                  wordWrap: 'on',
                  lineNumbers: 'on',
                  renderWhitespace: 'boundary',
                  scrollBeyondLastLine: false,
                }}
              />
            </div>
            <div className="preview markdown-preview preview-for-pdf" dangerouslySetInnerHTML={{ __html: marked(markdown) }} />
          </Split>
        ) : (
          <>
            {viewMode === 'edit' && (
              <div className="editor-container">
                <Editor
                  height="100%"
                  defaultLanguage="markdown"
                  value={markdown}
                  onChange={handleEditorChange}
                  onMount={handleEditorDidMount}
                  theme="vs-light"
                  options={{
                    minimap: { enabled: false },
                    fontSize: 14,
                    wordWrap: 'on',
                    lineNumbers: 'on',
                    renderWhitespace: 'boundary',
                    scrollBeyondLastLine: false,
                  }}
                />
              </div>
            )}
            {viewMode === 'preview' && (
              <div className="preview markdown-preview preview-for-pdf" dangerouslySetInnerHTML={{ __html: marked(markdown) }} />
            )}
          </>
        )}
      </div>
    </div>
  );
};

export default App;
